==> ScientificPython-2.6.1-cp35-cp35m-manylinux1_i686.whl: Scientific/TkWidgets/Utility.py <==
# Various useful small widgets
#
# Written by Konrad Hinsen <hinsen@cnrs-orleans.fr>
# Last revision: 2008-8-18
#

import tkinter, tkinter.dialog, tkinter.filedialog
import copy, os, string


class FilenameEntry(tkinter.Frame):

    """
    Filename entry widget

    A FilenameEntry widget consists of three parts: an identifying
    label, a text entry field for the filename, and a button labelled
    'browse' which call a file selection dialog box for picking a file
    name.
    """

    def __init__(self, master, text, pattern = '*', must_exist = True,
                 **attr):
        """
        @param master: the parent widget
        @param text: the label in front of the filename box
        @type text: C{str}
        @param pattern: the filename matching pattern that determines the
                        file list in the file selection dialog
        @type pattern: C{str}
        @param must_exist: if C{True}, allow only names of existing files
        @type must_exist: C{bool}
        """
        self.pattern = pattern
        self.must_exist = must_exist
        newattr = copy.copy(attr)
        newattr['text'] = text
        tkinter.Frame.__init__(self, master)
        apply(tkinter.Label, (self,), newattr).pack(side=tkinter.LEFT)
        self.filename = tkinter.StringVar()
        tkinter.Button(self, text="Browse...",
                       command=self.browse).pack(side=tkinter.RIGHT)
        newattr = copy.copy(attr)
        newattr['textvariable'] = self.filename
        entry = tkinter.Entry(*(self,), **newattr)
        entry.pack(side=tkinter.RIGHT, expand=1, fill=tkinter.X)
        entry.icursor("end")

    def browse(self):
        if self.must_exist:
            file = tkinter.filedialog.LoadFileDialog(self).go(pattern=self.pattern)
        else:
            file = tkinter.filedialog.SaveFileDialog(self).go(pattern=self.pattern)
        if file:
            self.filename.set(file)

    def get(self):
        """
        @returns: the current filename
        @rtype: C{str}
        @raises ValueError: if must_exist is C{True} and the name does not
                            refer to an existing file
        """
        filename =  self.filename.get()
        if self.must_exist and not os.path.exists(filename):
            tkinter.dialog.Dialog(self, title='File not found',
                          text='The file "' + filename + '" does not exist.',
                          bitmap='warning', default=0,
                          strings = ('Cancel',))
            raise ValueError
        return filename


class FloatEntry(tkinter.Frame):

    """
    Entry field for float numbers

    A FloatEntry widget consists of a label followed by a text entry
    field. 
    """
    
    def __init__(self, master, text, init = None, lower=None, upper=None,
                 name = None, **attr):
        """
        @param master: the parent widget
        @param text: the label in front of the entry field
        @type text: C{str}
        @param init: an optional initial value (default: blank field)
        @type init: number
        @param upper: an optional upper limit for the value
        @type upper: number
        @param lower: an optional lower limit for the value
        @type lower: number
        """
        self.text = text
        self.lower = lower
        self.upper = upper
        if name is None:
            name = text
        self.name = name
        newattr = copy.copy(attr)
        newattr['text'] = text
        tkinter.Frame.__init__(self, master)
        apply(tkinter.Label, (self,), newattr).pack(side=tkinter.LEFT)
        self.value = tkinter.DoubleVar()
        if init is not None:
            self.value.set(init)
        newattr = copy.copy(attr)
        newattr['textvariable'] = self.value
        self.entry = tkinter.Entry(*(self,), **newattr)
        self.entry.pack(side=tkinter.RIGHT, anchor=tkinter.E,
                        expand=1, fill=tkinter.X)
        self.entry.icursor("end")

    def bind(self, sequence=None, func=None, add=None):
        self.entry.bind(sequence, func, add)

    def set(self, value):
        """
        Set the value displayed in the field

        @param value: the new value
        @type value: C{float}
        """
        return self.value.set(value)

    def get(self):
        """
        @returns: the current value displayed in the field
        @rtype: C{float}
        @raises ValueError: if the current value is not a valid
                            number or is not within the specified limits
        """
        try:
            value = self.value.get()
        except (tkinter.TclError, ValueError):
            tkinter.dialog.Dialog(self, title='Illegal value',
                          text='The value of "' + self.name +
                               '" must be a number.',
                          bitmap='warning', default=0,
                          strings = ('Cancel',))
            raise ValueError
        range_check = 0
        if self.lower is not None and value < self.lower:
            range_check = -1
        if self.upper is not None and value > self.upper:
            range_check = 1
        if range_check != 0:
            text = 'The value of "' + self.name + '" must not be '
            if range_check < 0:
                text = text + 'smaller than ' + repr(self.lower) + '.'
            else:
                text = text + 'larger than ' + repr(self.upper) + '.'
            tkinter.dialog.Dialog(self, title='Value out of range', text=text,
                          bitmap='warning', default=0,
                          strings = ('Cancel',))
            raise ValueError
        return value


class IntEntry(FloatEntry):

    """
    Entry field for integer numbers

    A IntEntry widget consists of a label followed by a text entry
    field. 
    """
    
    def get(self):
        """
        @returns: the current value displayed in the field
        @rtype: C{int}
        @raises ValueError: if the current value is not a valid
                            number or is not within the specified limits
        """
        value = FloatEntry.get(self)
        ivalue = int(value)
        if ivalue != value:
            tkinter.dialog.Dialog(self, title='Illegal value',
                          text='The value of "' + self.name +
                               '" must be an integer.',
                          bitmap='warning', default=0,
                          strings = ('Cancel',))
            raise ValueError
        return ivalue

class ButtonBar(tkinter.Frame):

    """
    Horizontal array of buttons
    """

    def __init__(self, master, left_button_list, right_button_list):
        """
        @param master: the parent widget
        @param left_button_list: a list of (text, action) tuples specifying the
                                 buttons on the left-hand side of the button bar
        @param right_button_list: a list of (text, action) tuples specifying the
                                  buttons on the right-hand side of the button
                                  bar
        """
        tkinter.Frame.__init__(self, master, bd=2, relief=tkinter.SUNKEN)
        for button, action in left_button_list:
            tkinter.Button(self, text=button,
                           command=action).pack(side=tkinter.LEFT)
        for button, action in right_button_list:
            tkinter.Button(self, text=button,
                           command=action).pack(side=tkinter.RIGHT)


class StatusBar(tkinter.Frame):

    """
    Status bar

    A status bar can be used to inform the user about the status of an
    ongoing calculation. A message can be displayed with set() and
    removed with clear(). In both cases, the StatusBar object makes
    sure that the change takes place immediately. While a message
    is being displayed, the cursor form is changed to a watch.
    """

    def __init__(self, master):
        """
        @param master: the parent widget
        """
        tkinter.Frame.__init__(self, master, bd=2, relief=tkinter.RAISED)
        self.text = tkinter.Label(self, text='')
        self.text.pack(side=tkinter.LEFT, expand=tkinter.YES)

    def set(self, text):
        """
        Set a message to be displayed in the status bar
        
        @param text: the text of the message
        """
        self.text.configure(text = text)
        self.text.update_idletasks()
        self.master.configure(cursor='watch')
        self.update()
        self.update_idletasks()

    def clear(self):
        """
        Clear any message displayed in the status bar
        """
        self.text.configure(text = '')
        self.text.update_idletasks()
        self.master.configure(cursor='top_left_arrow')
        self.update_idletasks()


#
# The following class was taken from the Pythonware Tkinter introduction
#
class ModalDialog(tkinter.Toplevel):

    def __init__(self, parent, title = None):

        tkinter.Toplevel.__init__(self, parent)
        self.transient(parent)
        
        if title:
            self.title(title)

        self.parent = parent

        self.result = None

        body = tkinter.Frame(self)
        self.initial_focus = self.body(body)
        body.pack(padx=5, pady=5)

        self.buttonbox()

        self.grab_set()

        if not self.initial_focus:
            self.initial_focus = self

        self.protocol("WM_DELETE_WINDOW", self.cancel)

        self.geometry("+%d+%d" % (parent.winfo_rootx()+50,
                                  parent.winfo_rooty()+50))

        self.initial_focus.focus_set()

        self.wait_window(self)

    #
    # construction hooks

    def body(self, master):
        # create dialog body.  return widget that should have
        # initial focus.  this method should be overridden

        pass

    def buttonbox(self):
        # add standard button box. override if you don't want the
        # standard buttons

        box = tkinter.Frame(self)

        w = tkinter.Button(box, text="OK", width=10,
                           command=self.ok, default=tkinter.ACTIVE)
        w.pack(side=tkinter.LEFT, padx=5, pady=5)
        w = tkinter.Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=tkinter.LEFT, padx=5, pady=5)

        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)

        box.pack()

    #
    # standard button semantics

    def ok(self, event=None):

        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return

        self.withdraw()
        self.update_idletasks()

        self.apply()

        self.cancel()

    def cancel(self, event=None):

        # put focus back to the parent window
        self.parent.focus_set()
        self.destroy()

    #
    # command hooks

    def validate(self):

        return 1 # override

    def apply(self):

        pass # override


if __name__ == '__main__':
    
    class MyDialog(ModalDialog):

        def body(self, master):

            tkinter.Label(master, text="First:").grid(row=0)
            tkinter.Label(master, text="Second:").grid(row=1)

            self.e1 = IntEntry(master, '', 0, 0, 10, fg='red')
            self.e2 = tkinter.Entry(master)

            self.e1.grid(row=0, column=1)
            self.e2.grid(row=1, column=1)
            return self.e1 # initial focus

        def apply(self):
            first = string.atoi(self.e1.get())
            second = string.atoi(self.e2.get())
            self.result = first, second

    root = tkinter.Tk()
    tkinter.Button(root, text="Hello!").pack()
    root.update()
    d = MyDialog(root)
    print(d.result)

==> ScientificPython-2.6.3-cp35-cp35m-manylinux1_i686.whl: Scientific/TkWidgets/Utility.py <==
# Various useful small widgets
#
# Written by Konrad Hinsen <hinsen@cnrs-orleans.fr>
# Last revision: 2008-8-18
#

import tkinter, tkinter.dialog, tkinter.filedialog
import copy, os, string


class FilenameEntry(tkinter.Frame):

    """
    Filename entry widget

    A FilenameEntry widget consists of three parts: an identifying
    label, a text entry field for the filename, and a button labelled
    'browse' which call a file selection dialog box for picking a file
    name.
    """

    def __init__(self, master, text, pattern = '*', must_exist = True,
                 **attr):
        """
        @param master: the parent widget
        @param text: the label in front of the filename box
        @type text: C{str}
        @param pattern: the filename matching pattern that determines the
                        file list in the file selection dialog
        @type pattern: C{str}
        @param must_exist: if C{True}, allow only names of existing files
        @type must_exist: C{bool}
        """
        self.pattern = pattern
        self.must_exist = must_exist
        newattr = copy.copy(attr)
        newattr['text'] = text
        tkinter.Frame.__init__(self, master)
        apply(tkinter.Label, (self,), newattr).pack(side=tkinter.LEFT)
        self.filename = tkinter.StringVar()
        tkinter.Button(self, text="Browse...",
                       command=self.browse).pack(side=tkinter.RIGHT)
        newattr = copy.copy(attr)
        newattr['textvariable'] = self.filename
        entry = tkinter.Entry(*(self,), **newattr)
        entry.pack(side=tkinter.RIGHT, expand=1, fill=tkinter.X)
        entry.icursor("end")

    def browse(self):
        if self.must_exist:
            file = tkinter.filedialog.LoadFileDialog(self).go(pattern=self.pattern)
        else:
            file = tkinter.filedialog.SaveFileDialog(self).go(pattern=self.pattern)
        if file:
            self.filename.set(file)

    def get(self):
        """
        @returns: the current filename
        @rtype: C{str}
        @raises ValueError: if must_exist is C{True} and the name does not
                            refer to an existing file
        """
        filename =  self.filename.get()
        if self.must_exist and not os.path.exists(filename):
            tkinter.dialog.Dialog(self, title='File not found',
                          text='The file "' + filename + '" does not exist.',
                          bitmap='warning', default=0,
                          strings = ('Cancel',))
            raise ValueError
        return filename


class FloatEntry(tkinter.Frame):

    """
    Entry field for float numbers

    A FloatEntry widget consists of a label followed by a text entry
    field. 
    """
    
    def __init__(self, master, text, init = None, lower=None, upper=None,
                 name = None, **attr):
        """
        @param master: the parent widget
        @param text: the label in front of the entry field
        @type text: C{str}
        @param init: an optional initial value (default: blank field)
        @type init: number
        @param upper: an optional upper limit for the value
        @type upper: number
        @param lower: an optional lower limit for the value
        @type lower: number
        """
        self.text = text
        self.lower = lower
        self.upper = upper
        if name is None:
            name = text
        self.name = name
        newattr = copy.copy(attr)
        newattr['text'] = text
        tkinter.Frame.__init__(self, master)
        apply(tkinter.Label, (self,), newattr).pack(side=tkinter.LEFT)
        self.value = tkinter.DoubleVar()
        if init is not None:
            self.value.set(init)
        newattr = copy.copy(attr)
        newattr['textvariable'] = self.value
        self.entry = tkinter.Entry(*(self,), **newattr)
        self.entry.pack(side=tkinter.RIGHT, anchor=tkinter.E,
                        expand=1, fill=tkinter.X)
        self.entry.icursor("end")

    def bind(self, sequence=None, func=None, add=None):
        self.entry.bind(sequence, func, add)

    def set(self, value):
        """
        Set the value displayed in the field

        @param value: the new value
        @type value: C{float}
        """
        return self.value.set(value)

    def get(self):
        """
        @returns: the current value displayed in the field
        @rtype: C{float}
        @raises ValueError: if the current value is not a valid
                            number or is not within the specified limits
        """
        try:
            value = self.value.get()
        except (tkinter.TclError, ValueError):
            tkinter.dialog.Dialog(self, title='Illegal value',
                          text='The value of "' + self.name +
                               '" must be a number.',
                          bitmap='warning', default=0,
                          strings = ('Cancel',))
            raise ValueError
        range_check = 0
        if self.lower is not None and value < self.lower:
            range_check = -1
        if self.upper is not None and value > self.upper:
            range_check = 1
        if range_check != 0:
            text = 'The value of "' + self.name + '" must not be '
            if range_check < 0:
                text = text + 'smaller than ' + repr(self.lower) + '.'
            else:
                text = text + 'larger than ' + repr(self.upper) + '.'
            tkinter.dialog.Dialog(self, title='Value out of range', text=text,
                          bitmap='warning', default=0,
                          strings = ('Cancel',))
            raise ValueError
        return value


class IntEntry(FloatEntry):

    """
    Entry field for integer numbers

    A IntEntry widget consists of a label followed by a text entry
    field. 
    """
    
    def get(self):
        """
        @returns: the current value displayed in the field
        @rtype: C{int}
        @raises ValueError: if the current value is not a valid
                            number or is not within the specified limits
        """
        value = FloatEntry.get(self)
        ivalue = int(value)
        if ivalue != value:
            tkinter.dialog.Dialog(self, title='Illegal value',
                          text='The value of "' + self.name +
                               '" must be an integer.',
                          bitmap='warning', default=0,
                          strings = ('Cancel',))
            raise ValueError
        return ivalue

class ButtonBar(tkinter.Frame):

    """
    Horizontal array of buttons
    """

    def __init__(self, master, left_button_list, right_button_list):
        """
        @param master: the parent widget
        @param left_button_list: a list of (text, action) tuples specifying the
                                 buttons on the left-hand side of the button bar
        @param right_button_list: a list of (text, action) tuples specifying the
                                  buttons on the right-hand side of the button
                                  bar
        """
        tkinter.Frame.__init__(self, master, bd=2, relief=tkinter.SUNKEN)
        for button, action in left_button_list:
            tkinter.Button(self, text=button,
                           command=action).pack(side=tkinter.LEFT)
        for button, action in right_button_list:
            tkinter.Button(self, text=button,
                           command=action).pack(side=tkinter.RIGHT)


class StatusBar(tkinter.Frame):

    """
    Status bar

    A status bar can be used to inform the user about the status of an
    ongoing calculation. A message can be displayed with set() and
    removed with clear(). In both cases, the StatusBar object makes
    sure that the change takes place immediately. While a message
    is being displayed, the cursor form is changed to a watch.
    """

    def __init__(self, master):
        """
        @param master: the parent widget
        """
        tkinter.Frame.__init__(self, master, bd=2, relief=tkinter.RAISED)
        self.text = tkinter.Label(self, text='')
        self.text.pack(side=tkinter.LEFT, expand=tkinter.YES)

    def set(self, text):
        """
        Set a message to be displayed in the status bar
        
        @param text: the text of the message
        """
        self.text.configure(text = text)
        self.text.update_idletasks()
        self.master.configure(cursor='watch')
        self.update()
        self.update_idletasks()

    def clear(self):
        """
        Clear any message displayed in the status bar
        """
        self.text.configure(text = '')
        self.text.update_idletasks()
        self.master.configure(cursor='top_left_arrow')
        self.update_idletasks()


#
# The following class was taken from the Pythonware Tkinter introduction
#
class ModalDialog(tkinter.Toplevel):

    def __init__(self, parent, title = None):

        tkinter.Toplevel.__init__(self, parent)
        self.transient(parent)
        
        if title:
            self.title(title)

        self.parent = parent

        self.result = None

        body = tkinter.Frame(self)
        self.initial_focus = self.body(body)
        body.pack(padx=5, pady=5)

        self.buttonbox()

        self.grab_set()

        if not self.initial_focus:
            self.initial_focus = self

        self.protocol("WM_DELETE_WINDOW", self.cancel)

        self.geometry("+%d+%d" % (parent.winfo_rootx()+50,
                                  parent.winfo_rooty()+50))

        self.initial_focus.focus_set()

        self.wait_window(self)

    #
    # construction hooks

    def body(self, master):
        # create dialog body.  return widget that should have
        # initial focus.  this method should be overridden

        pass

    def buttonbox(self):
        # add standard button box. override if you don't want the
        # standard buttons

        box = tkinter.Frame(self)

        w = tkinter.Button(box, text="OK", width=10,
                           command=self.ok, default=tkinter.ACTIVE)
        w.pack(side=tkinter.LEFT, padx=5, pady=5)
        w = tkinter.Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=tkinter.LEFT, padx=5, pady=5)

        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)

        box.pack()

    #
    # standard button semantics

    def ok(self, event=None):

        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return

        self.withdraw()
        self.update_idletasks()

        self.apply()

        self.cancel()

    def cancel(self, event=None):

        # put focus back to the parent window
        self.parent.focus_set()
        self.destroy()

    #
    # command hooks

    def validate(self):

        return 1 # override

    def apply(self):

        pass # override


if __name__ == '__main__':
    
    class MyDialog(ModalDialog):

        def body(self, master):

            tkinter.Label(master, text="First:").grid(row=0)
            tkinter.Label(master, text="Second:").grid(row=1)

            self.e1 = IntEntry(master, '', 0, 0, 10, fg='red')
            self.e2 = tkinter.Entry(master)

            self.e1.grid(row=0, column=1)
            self.e2.grid(row=1, column=1)
            return self.e1 # initial focus

        def apply(self):
            first = int(self.e1.get())
            second = int(self.e2.get())
            self.result = first, second

    root = tkinter.Tk()
    tkinter.Button(root, text="Hello!").pack()
    root.update()
    d = MyDialog(root)
    print(d.result)
